-- $Id: test_two.adb,v 1.1 2008/11/20 22:18:50 baker Exp baker $

--  This is a second test program for simulation packages.
--  It tests one periodic and one aperiodic task.

with Ada.Exceptions;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Integer_Text_IO; use Ada.Integer_Text_IO;
with Ada.Float_Text_IO; use Ada.Float_Text_IO;
with Ada.Strings; use Ada.Strings;
with Ada.Strings.Fixed; use Ada.Strings.Fixed;
with Error_Log; use Error_Log;
with Tasks;
with Simulator;
with Threads;
with Threads.Sched_DSS;
with Workload_Models;
with Aperiodic_Server_Parameters; use Aperiodic_Server_Parameters;
with Aperiodic_Workloads;
with Random_Tools;
with Stats;
with Periodic_Workloads;
with Threads.Sched_EDF;

procedure Test_Two is

   --  Aperiodic tasks

   subtype AP_Range is Integer range 1 .. 1;

   Aperiodic_Tasks : array (AP_Range) of aliased Tasks.Task_Object;

   Aperiodic_Loads : array (AP_Range) of aliased Aperiodic_Workloads.Object;

   Aperiodic_Policies : array (AP_Range)
     of aliased Threads.Sched_DSS.Object;

   Aperiodic_Load_Parameters : array (AP_Range)
     of aliased Aperiodic_Workloads.Parameters :=
     (1 => (Mean_Interarrival_Time => 3.0, Mean_Execution_Time => 2.0));

   Aperiodic_Policy_Parameters : array (AP_Range)
     of Aperiodic_Server_Parameters.Parameters :=
     (1 => (Budget => 1, Budget_Interval => 2));

   --  Periodic tasks

   subtype P_Range is Integer range 1 .. 1;

   Periodic_tasks : array (P_Range) of aliased Tasks.Task_Object;

   Periodic_Loads : array (P_Range) of aliased Periodic_Workloads.Object;

   Periodic_Load_Parameters : array (P_Range)
     of aliased Periodic_Workloads.Parameters :=
     (1 => (Period => 5, Deadline => 4, WCET => 2));

   Periodic_Policies : array (P_Range)
     of aliased Threads.Sched_EDF.Object;

   Test_Name : constant String := "test_two";

begin

   Error_Log.Open (Test_Name);
   Error_Log.Debug_Level := 3;

   --  run just one simulation, for now

   --  set up one workload model object and one
   --  scheduling policy object per aperiodic task

   for K in Aperiodic_Tasks'Range loop
      Aperiodic_Loads (K).Bind_Parms (Aperiodic_Load_Parameters (K));
      Aperiodic_Policies (K).Bind_Parms (Aperiodic_Policy_Parameters (K));
      Tasks.Bind
        (Aperiodic_Tasks (K)'Unchecked_Access,
         M => Aperiodic_Loads (K)'Unchecked_Access,
         P => Aperiodic_Policies (K)'Unchecked_Access,
         Name => 'A' & Trim (Integer'Image (K),Left));
   end loop;

   for K in Periodic_Tasks'Range loop
      Periodic_Loads (K).Bind_Parms (Periodic_Load_Parameters (K));
      Tasks.Bind
        (Periodic_Tasks (K)'Unchecked_Access,
         M => Periodic_Loads (K)'Unchecked_Access,
         P => Periodic_Policies (K)'Unchecked_Access,
         Name => 'P' & Trim (Integer'Image (K + Periodic_Tasks'Last), Left));
   end loop;

   --  start the pseudo-random number generator that
   --  determines the "random" events in the simulation

   Random_Tools.Start (3, 5, 7, 11, False);

   --  initialize/reset the tasks,
   --  and thereby everything that depends on them

   Tasks.Initialize;
   Simulator.Run (10);

   for K in Aperiodic_Tasks'Range loop
      declare
         S : Stats.Data := Tasks.Get_Stats (Aperiodic_Tasks (K)'Unchecked_Access);
      begin
         --  just the basics, for now
         Put ("Max_Response_Time = ");
         Put (Integer (S.Max_Response_Time));
         New_Line;
         Put ("Total_Response_Time = ");
         Put (Integer (S.Total_Response_Time));
         New_Line;
         Put ("Job_Count = ");
         Put (Integer (S.Job_Count));
         New_Line;
         Put ("Missed_Deadlines = ");
         Put (Integer (S.Missed_Deadlines));
         New_Line;
      end;
   end loop;

   for K in Periodic_Tasks'Range loop
      declare
         S : Stats.Data := Tasks.Get_Stats (Periodic_Tasks (K)'Unchecked_Access);
      begin
         --  just the basics, for now
         Put ("Max_Response_Time = ");
         Put (Integer (S.Max_Response_Time));
         New_Line;
         Put ("Total_Response_Time = ");
         Put (Integer (S.Total_Response_Time));
         New_Line;
         Put ("Job_Count = ");
         Put (Integer (S.Job_Count));
         New_Line;
         Put ("Missed_Deadlines = ");
         Put (Integer (S.Missed_Deadlines));
         New_Line;
      end;
   end loop;

   Error_Log.Close;

exception
   when E : others =>
      Put (Log, "exception " & Ada.Exceptions.Exception_Name (E));
      Put (Log, " in " & Test_Name & " at ");
      Put_Line (Log, Ada.Exceptions.Exception_Message (E));
      Error_Log.Close;
      raise;
end Test_Two;
